      subroutine twoemm(nreal,ninteger,nconts,ncontp,ncontd,ncds,ncdspd,
     &                  nxcs,nxcspd,nshels,nshelp,nsheld,mtloca,iprimvc,
     &                  nprimvc,icentvc,icdlfunc,icdcfunc,ixclfunc,
     &                  ixccfunc,iwkvec,cutoff,coord,alpha,coeff,
     &                  alphacd,coefscd,coefpcd,coefdcd,alphaxc,coefsxc,
     &                  coefpxc,coefdxc,cdfitc,xcfit1,xcfit2,xcfit3,
     &                  fock,fock1,fock2,fock3,density,wkvec)


c  this subroutine written by alain st-amant of the department of
c  pharmaceutical chemistry, university of california, san francisco.
c  all rights reserved.  this is part of the DeFT project.

      implicit real*8(a-h,o-z)

      include "mpif.h"

      dimension jsend(20)

      dimension istat(mpi_status_size)

      dimension g(7)

      dimension nshels(*),nshelp(*),nsheld(*),mtloca(*),iprimvc(*),
     &          nprimvc(*),icentvc(*),icdlfunc(*),icdcfunc(*),
     &          ixclfunc(*),ixccfunc(*),iwkvec(*)

      dimension coord(3,*),alpha(*),coeff(*),alphacd(*),coefscd(*),
     &          coefpcd(*),coefdcd(*),alphaxc(*),coefsxc(*),coefpxc(*),
     &          coefdxc(*),cdfitc(*),xcfit1(*),xcfit2(*),xcfit3(*),
     &          fock(*),fock1(*),fock2(*),fock3(*),density(*),wkvec(*)

      data zero,one/0.0d0,1.0d0/

      call mpi_comm_rank(mpi_comm_world,myid,ierr)
      call mpi_comm_size(mpi_comm_world,numprocs,ierr)

      call igftable

      ncdfunc=ncds+10*ncdspd
      nxcfunc=nxcs+10*nxcspd

      call mpi_bcast(cdfitc,ncdfunc,mpi_real8,0,mpi_comm_world,ierr)
      call mpi_bcast(xcfit1,nxcfunc,mpi_real8,0,mpi_comm_world,ierr)
      call mpi_bcast(xcfit2,nxcfunc,mpi_real8,0,mpi_comm_world,ierr)
      call mpi_bcast(xcfit3,nxcfunc,mpi_real8,0,mpi_comm_world,ierr)

      ndim=nconts+3*ncontp+6*ncontd
      ndim=ndim*(ndim+1)/2

      do 1001 i=1,ndim
      fock(i)=zero
      density(i)=one
 1001 if(nxcfunc.gt.0) fock3(i)=zero

      ncposs=nconts*(nconts+1)/2
      ncpopp=ncontp*(ncontp+1)/2
      ncpodd=ncontd*(ncontd+1)/2

      ncpops=ncontp*nconts
      ncpods=ncontd*nconts
      ncpodp=ncontd*ncontp

      npposs=0
      nppops=0
      nppopp=0
      nppods=0
      nppodp=0
      nppodd=0

      do 1002 i=1,nconts
      do 1002 j=1,i
 1002 npposs=npposs+nprimvc(nshels(i))*nprimvc(nshels(j))

      do 1003 i=1,ncontp
      do 1003 j=1,nconts
 1003 nppops=nppops+nprimvc(nshelp(i))*nprimvc(nshels(j))

      do 1004 i=1,ncontp
      do 1004 j=1,i
 1004 nppopp=nppopp+nprimvc(nshelp(i))*nprimvc(nshelp(j))

      do 1005 i=1,ncontd
      do 1005 j=1,nconts
 1005 nppods=nppods+nprimvc(nsheld(i))*nprimvc(nshels(j))

      do 1006 i=1,ncontd
      do 1006 j=1,ncontp
 1006 nppodp=nppodp+nprimvc(nsheld(i))*nprimvc(nshelp(j))

      do 1007 i=1,ncontd
      do 1007 j=1,i
 1007 nppodd=nppodd+nprimvc(nsheld(i))*nprimvc(nsheld(j))

      ja=1
      jb=ja+npposs
      jc=jb+ncposs

      if(jc.gt.ninteger) stop 'twoemm ss integer overflow'

      je=1
      jf=je+npposs
      jg=jf+npposs
      jh=jg+npposs
      ji=jh+npposs
      jj=ji+npposs
      jk=jj+npposs
      jl=jk+npposs
      jm=jl+npposs
      jn=jm+npposs
      jo=jn+npposs
      jp=jo+npposs
      jq=jp+npposs
      jr=jq+npposs*8

      if(jr.gt.nreal) stop 'twoemm ss real overflow'

      call setupcdss(ncposs,n,nconts,nshels,mtloca,iprimvc,nprimvc,
     &               icentvc,iwkvec(ja),iwkvec(jb),cutoff,coord,alpha,
     &               coeff,wkvec(je),wkvec(jf),wkvec(jg),wkvec(jh),
     &               wkvec(ji),wkvec(jj),wkvec(jk),wkvec(jl),wkvec(jm),
     &               wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jq),density)

      jd=jc+n
      j0=jd+n

      if(j0.gt.ninteger) stop 'twoemm ss integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                    endif

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n

      if(myid.eq.0) then
                          isent=0

                          do 1008 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=ja
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp

                          call mpi_bsend(jsend,13,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(ja),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          ja=ja+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop

 1008                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,13,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ja=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)

      call mpi_recv(iwkvec(ja),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jq+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+ncposs
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+ncposs*3
      kl=kk+nloop*3
      km=kl+nloop*3
      kn=km+ncposs*6
      ko=kn+nloop*6
      kp=ko+nloop
      kq=kp+nloop
      kr=kq+nloop
      ks=kr+nloop
      kt=ks+nloop
      ku=kt+nloop
      kv=ku+nloop

      if(kv.gt.nreal) stop 'twoemm ss real'

      call chrgss(ncposs,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jc),iwkvec(jb),iwkvec(jd),g,coord,cdfitc,
     &            alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jm),wkvec(ka),
     &            wkvec(kb),wkvec(kc),wkvec(kd),wkvec(ke),wkvec(kf),
     &            wkvec(kg),wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &            wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),wkvec(kp),
     &            wkvec(kq),wkvec(kr),wkvec(ks),wkvec(kt),wkvec(ku))

      ka=jq+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop*3
      kk=kj+nloop*3
      kl=kk+nloop*6
      km=kl+nloop*6
      kn=km+nloop*3
      ko=kn+nloop*9
      kp=ko+nloop*6
      kq=kp+nloop
      kr=kq+nloop
      ks=kr+nloop
      kt=ks+nloop
      ku=kt+nloop
      kv=ku+nloop
      kw=kv+nloop
      kx=kw+nloop
      ky=kx+nloop
      kz=ky+nloop
      la=kz+nloop
      lb=la+nloop*3

      if(nxcfunc.gt.0) then

                 if(la.gt.nreal) stop 'twoemm ss real'

                 call xcorss(ncposs,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jc),iwkvec(jb),
     &                       iwkvec(jd),coord,xcfit1,xcfit2,xcfit3,
     &                       alphaxc,coefsxc,coefpxc,coefdxc,fock1,
     &                       fock2,fock3,wkvec(je),wkvec(jh),wkvec(ji),
     &                       wkvec(jj),wkvec(jn),wkvec(jo),wkvec(jp),
     &                       wkvec(jm),wkvec(ka),wkvec(kb),wkvec(kc),
     &                       wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),
     &                       wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &                       wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &                       wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),
     &                       wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &                       wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la))

                       endif

      if(ncontp.eq.0) goto 2001

      ja=1
      jb=ja+nppops
      jc=jb+ncpops*3

      if(jc.gt.ninteger) stop 'twoemm ps integer overflow'

      je=1
      jf=je+nppops
      jg=jf+nppops
      jh=jg+nppops
      ji=jh+nppops
      jj=ji+nppops
      jk=jj+nppops
      jl=jk+nppops
      jm=jl+nppops
      jn=jm+nppops
      jo=jn+nppops
      jp=jo+nppops
      jq=jp+nppops
      jr=jq+nppops
      js=jr+nppops
      jt=js+nppops
      ju=jt+nppops*11

      if(ju.gt.nreal) stop 'twoemm ps real overflow'

      call setupcdps(ncpops,n,ncontp,nconts,nshelp,nshels,mtloca,
     &               iprimvc,nprimvc,icentvc,iwkvec(ja),iwkvec(jb),
     &               cutoff,coord,alpha,coeff,wkvec(je),wkvec(jf),
     &               wkvec(jg),wkvec(jh),wkvec(ji),wkvec(jj),wkvec(jk),
     &               wkvec(jl),wkvec(jm),wkvec(jn),wkvec(jo),wkvec(jp),
     &               wkvec(jq),wkvec(jr),wkvec(js),wkvec(jt),density)

      jd=jc+n
      j0=jd+n

      if(j0.gt.ninteger) stop 'twoemm ps integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                          call stringcopy(n,wkvec(12*n+1),wkvec(jq))
                          call stringcopy(n,wkvec(13*n+1),wkvec(jr))
                          call stringcopy(n,wkvec(14*n+1),wkvec(js))
                    endif

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n
      jr=jq+n
      js=jr+n
      jt=js+n

      if(myid.eq.0) then
                          isent=0

                          do 1009 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=ja
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp
                          jsend(14)=jq
                          jsend(15)=jr
                          jsend(16)=js

                          call mpi_bsend(jsend,16,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(ja),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jq),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jr),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(js),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          ja=ja+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop
                          jq=jq+nloop
                          jr=jr+nloop
                          js=js+nloop

 1009                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,16,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ja=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)
      jq=jsend(14)
      jr=jsend(15)
      js=jsend(16)

      call mpi_recv(iwkvec(ja),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jq),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jr),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(js),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jt+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+nloop
      kl=kk+nloop
      km=kl+nloop
      kn=km+ncpops*3
      ko=kn+nloop*3
      kp=ko+nloop*3
      kq=kp+nloop*3
      kr=kq+nloop*3
      ks=kr+ncpops*9
      kt=ks+nloop*9
      ku=kt+nloop*9
      kv=ku+ncpops*18
      kw=kv+nloop*18
      kx=kw+nloop
      ky=kx+nloop
      kz=ky+nloop
      la=kz+nloop*3
      lb=la+nloop*3
      lc=lb+nloop
      ld=lc+nloop
      le=ld+nloop
      lf=le+nloop
      lg=lf+nloop
      lh=lg+nloop
      li=lh+nloop
      lj=li+nloop*3

      if(lj.gt.nreal) stop 'twoemm ps real'

      call chrgps(ncpops,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jc),iwkvec(jb),iwkvec(jd),g,coord,cdfitc,
     &            alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jm),wkvec(jq),
     &            wkvec(jr),wkvec(js),wkvec(ka),wkvec(kb),wkvec(kc),
     &            wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),wkvec(kh),
     &            wkvec(ki),wkvec(kj),wkvec(kk),wkvec(kl),wkvec(km),
     &            wkvec(kn),wkvec(ko),wkvec(kp),wkvec(kq),wkvec(kr),
     &            wkvec(ks),wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &            wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),wkvec(lb),
     &            wkvec(lc),wkvec(ld),wkvec(le),wkvec(lf),wkvec(lg),
     &            wkvec(lh),wkvec(li))

      ka=jt+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop*3
      kj=ki+nloop*3
      kk=kj+nloop*6
      kl=kk+nloop*6
      km=kl+nloop*10
      kn=km+nloop*10
      ko=kn+nloop*9
      kp=ko+nloop*18
      kq=kp+nloop*18
      kr=kq+nloop
      ks=kr+nloop*3
      kt=ks+nloop
      ku=kt+nloop
      kv=ku+nloop
      kw=kv+nloop
      kx=kw+nloop
      ky=kx+nloop
      kz=ky+nloop
      la=kz+nloop
      lb=la+nloop
      lc=lb+nloop
      ld=lc+nloop*9

      if(nxcfunc.gt.0) then

                 if(ld.gt.nreal) stop 'twoemm ps real'

                 call xcorps(ncpops,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jc),iwkvec(jb),
     &                       iwkvec(jd),coord,xcfit1,xcfit2,xcfit3,
     &                       alphaxc,coefsxc,coefpxc,coefdxc,fock1,
     &                       fock2,fock3,wkvec(je),wkvec(jh),wkvec(ji),
     &                       wkvec(jj),wkvec(jn),wkvec(jo),wkvec(jp),
     &                       wkvec(jm),wkvec(ka),wkvec(kb),wkvec(kc),
     &                       wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),
     &                       wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &                       wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &                       wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),
     &                       wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &                       wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),
     &                       wkvec(lb),wkvec(lc))

                       endif

      ja=1
      jb=ja+ncpopp
      jc=jb+nppopp
      jd=jc+ncpopp*9

      if(jd.gt.ninteger) stop 'twoemm pp integer overflow'

      je=1
      jf=je+nppopp
      jg=jf+nppopp
      jh=jg+nppopp
      ji=jh+nppopp
      jj=ji+nppopp
      jk=jj+nppopp
      jl=jk+nppopp
      jm=jl+nppopp
      jn=jm+nppopp
      jo=jn+nppopp
      jp=jo+nppopp
      jq=jp+nppopp
      jr=jq+nppopp
      js=jr+nppopp
      jt=js+nppopp
      ju=jt+nppopp
      jv=ju+nppopp
      jw=jv+nppopp
      jx=jw+ncpopp
      jy=jx+ncpopp
      jz=jy+ncpopp
      j0=jz+nppopp*11

      if(j0.gt.nreal) stop 'twoemm pp real overflow'

      call setupcdpp(ncpopp,n,ncontp,nshelp,mtloca,iprimvc,nprimvc,
     &               icentvc,iwkvec(ja),iwkvec(jb),iwkvec(jc),cutoff,
     &               coord,alpha,coeff,wkvec(je),wkvec(jf),wkvec(jg),
     &               wkvec(jh),wkvec(ji),wkvec(jj),wkvec(jk),wkvec(jl),
     &               wkvec(jm),wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jq),
     &               wkvec(jr),wkvec(js),wkvec(jt),wkvec(ju),wkvec(jv),
     &               wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jz),density)

      j0=jd+n
      j1=j0+n

      if(j1.gt.ninteger) stop 'twoemm pp integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                          call stringcopy(n,wkvec(12*n+1),wkvec(jq))
                          call stringcopy(n,wkvec(13*n+1),wkvec(jr))
                          call stringcopy(n,wkvec(14*n+1),wkvec(js))
                          call stringcopy(n,wkvec(15*n+1),wkvec(jt))
                          call stringcopy(n,wkvec(16*n+1),wkvec(ju))
                          call stringcopy(n,wkvec(17*n+1),wkvec(jv))
                    endif

      call stringcopy(ncpopp,wkvec(18*n+0*ncpopp+1),wkvec(jw))
      call stringcopy(ncpopp,wkvec(18*n+1*ncpopp+1),wkvec(jx))
      call stringcopy(ncpopp,wkvec(18*n+2*ncpopp+1),wkvec(jy))

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n
      jr=jq+n
      js=jr+n
      jt=js+n
      ju=jt+n
      jv=ju+n
      jw=jv+n
      jx=jw+ncpopp
      jy=jx+ncpopp
      jz=jy+ncpopp

      if(myid.eq.0) then
                          isent=0

                          do 1010 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=jb
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp
                          jsend(14)=jq
                          jsend(15)=jr
                          jsend(16)=js
                          jsend(17)=jt
                          jsend(18)=ju
                          jsend(19)=jv

                          call mpi_bsend(jsend,19,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(jb),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jq),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jr),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(js),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          jb=jb+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop
                          jq=jq+nloop
                          jr=jr+nloop
                          js=js+nloop
                          jt=jt+nloop
                          ju=ju+nloop
                          jv=jv+nloop

 1010                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer, mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,19,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      jb=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)
      jq=jsend(14)
      jr=jsend(15)
      js=jsend(16)
      jt=jsend(17)
      ju=jsend(18)
      jv=jsend(19)

      call mpi_recv(iwkvec(jb),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jq),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jr),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(js),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+nloop
      kl=kk+nloop
      km=kl+nloop
      kn=km+nloop
      ko=kn+nloop*3
      kp=ko+nloop*3
      kq=kp+nloop*3
      kr=kq+nloop*3
      ks=kr+nloop*3
      kt=ks+nloop*6
      ku=kt+nloop*6
      kv=ku+nloop*6
      kw=kv+nloop*6
      kx=kw+nloop*3
      ky=kx+nloop*9
      kz=ky+nloop*9
      la=kz+nloop*9
      lb=la+nloop*18
      lc=lb+nloop*18
      ld=lc+nloop*18
      le=ld+nloop*18
      lf=le+nloop*18
      lg=lf+nloop*36
      lh=lg+nloop*36
      li=lh+nloop*9
      lj=li+nloop*27
      lk=lj+nloop*54
      ll=lk+nloop
      lm=ll+nloop
      ln=lm+nloop
      lo=ln+nloop*3
      lp=lo+nloop*6
      lq=lp+nloop*9
      lr=lq+nloop
      ls=lr+nloop
      lt=ls+nloop
      lu=lt+nloop
      lv=lu+nloop
      lw=lv+nloop
      lx=lw+nloop
      ly=lx+nloop*9

      if(ly.gt.nreal) stop 'twoemm pp real'

      call chrgpp(ncpopp,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jb),iwkvec(jd),iwkvec(jc),iwkvec(j0),g,coord,
     &            cdfitc,alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jn),wkvec(jo),
     &            wkvec(jp),wkvec(jm),wkvec(jq),wkvec(jr),wkvec(js),
     &            wkvec(ka),wkvec(kb),wkvec(kc),wkvec(kd),wkvec(ke),
     &            wkvec(kf),wkvec(kg),wkvec(kh),wkvec(ki),wkvec(kj),
     &            wkvec(kk),wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &            wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),wkvec(kt),
     &            wkvec(ku),wkvec(kv),wkvec(kw),wkvec(kx),wkvec(ky),
     &            wkvec(kz),wkvec(la),wkvec(lb),wkvec(lc),wkvec(ld),
     &            wkvec(le),wkvec(lf),wkvec(lg),wkvec(lh),wkvec(li),
     &            wkvec(lj),wkvec(lk),wkvec(ll),wkvec(lm),wkvec(ln),
     &            wkvec(lo),wkvec(lp),wkvec(lq),wkvec(lr),wkvec(ls),
     &            wkvec(lt),wkvec(lu),wkvec(lv),wkvec(lw),wkvec(lx))

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop*3
      kj=ki+nloop*3
      kk=kj+nloop*6
      kl=kk+nloop*6
      km=kl+nloop*10
      kn=km+nloop*10
      ko=kn+nloop*15
      kp=ko+nloop*15
      kq=kp+nloop*9
      kr=kq+nloop*18
      ks=kr+nloop*30
      kt=ks+nloop*27
      ku=kt+nloop*54
      kv=ku+nloop*54
      kw=kv+nloop
      kx=kw+nloop*3
      ky=kx+nloop*6
      kz=ky+nloop
      la=kz+nloop
      lb=la+nloop
      lc=lb+nloop
      ld=lc+nloop
      le=ld+nloop
      lf=le+nloop
      lg=lf+nloop
      lh=lg+nloop
      li=lh+nloop
      lj=li+nloop*27

      if(nxcfunc.gt.0) then

                 if(lj.gt.nreal) stop 'twoemm pp real'

                 call xcorpp(ncpopp,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jb),iwkvec(jd),
     &                       iwkvec(jc),iwkvec(j0),coord,xcfit1,xcfit2,
     &                       xcfit3,alphaxc,coefsxc,coefpxc,coefdxc,
     &                       fock1,fock2,fock3,wkvec(je),wkvec(jh),
     &                       wkvec(ji),wkvec(jj),wkvec(jw),wkvec(jx),
     &                       wkvec(jy),wkvec(jn),wkvec(jo),wkvec(jp),
     &                       wkvec(jm),wkvec(ka),wkvec(kb),wkvec(kc),
     &                       wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),
     &                       wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &                       wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &                       wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),
     &                       wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &                       wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),
     &                       wkvec(lb),wkvec(lc),wkvec(ld),wkvec(le),
     &                       wkvec(lf),wkvec(lg),wkvec(lh),wkvec(li))

                       endif

      if(ncontd.eq.0) goto 2001

      ja=1
      jb=ja+nppods
      jc=jb+ncpods*6

      if(jc.gt.ninteger) stop 'twoemm ds integer overflow'

      je=1
      jf=je+nppods
      jg=jf+nppods
      jh=jg+nppods
      ji=jh+nppods
      jj=ji+nppods
      jk=jj+nppods
      jl=jk+nppods
      jm=jl+nppods
      jn=jm+nppods
      jo=jn+nppods
      jp=jo+nppods
      jq=jp+nppods
      jr=jq+nppods
      js=jr+nppods
      jt=js+nppods
      ju=jt+nppods*11

      if(ju.gt.nreal) stop 'twoemm ds real overflow'

      call setupcdds(ncpods,n,ncontd,nconts,nsheld,nshels,mtloca,
     &               iprimvc,nprimvc,icentvc,iwkvec(ja),iwkvec(jb),
     &               cutoff,coord,alpha,coeff,wkvec(je),wkvec(jf),
     &               wkvec(jg),wkvec(jh),wkvec(ji),wkvec(jj),wkvec(jk),
     &               wkvec(jl),wkvec(jm),wkvec(jn),wkvec(jo),wkvec(jp),
     &               wkvec(jq),wkvec(jr),wkvec(js),wkvec(jt),density)

      jd=jc+n
      j0=jd+n

      if(j0.gt.ninteger) stop 'twoemm ds integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                          call stringcopy(n,wkvec(12*n+1),wkvec(jq))
                          call stringcopy(n,wkvec(13*n+1),wkvec(jr))
                          call stringcopy(n,wkvec(14*n+1),wkvec(js))
                    endif

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n
      jr=jq+n
      js=jr+n
      jt=js+n

      if(myid.eq.0) then
                          isent=0

                          do 1011 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=ja
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp
                          jsend(14)=jq
                          jsend(15)=jr
                          jsend(16)=js

                          call mpi_bsend(jsend,16,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(ja),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jq),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jr),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(js),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          ja=ja+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop
                          jq=jq+nloop
                          jr=jr+nloop
                          js=js+nloop

 1011                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,16,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ja=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)
      jq=jsend(14)
      jr=jsend(15)
      js=jsend(16)

      call mpi_recv(iwkvec(ja),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jq),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jr),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(js),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jt+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+nloop
      kl=kk+nloop
      km=kl+nloop
      kn=km+nloop
      ko=kn+nloop*3
      kp=ko+nloop*3
      kq=kp+nloop*3
      kr=kq+nloop*3
      ks=kr+ncpods*6
      kt=ks+nloop*6
      ku=kt+nloop*6
      kv=ku+nloop*6
      kw=kv+nloop*9
      kx=kw+ncpods*18
      ky=kx+nloop*18
      kz=ky+nloop*18
      la=kz+ncpods*36
      lb=la+nloop*36
      lc=lb+nloop
      ld=lc+nloop
      le=ld+nloop
      lf=le+nloop*3
      lg=lf+nloop*6
      lh=lg+nloop*9
      li=lh+nloop
      lj=li+nloop
      lk=lj+nloop
      ll=lk+nloop
      lm=ll+nloop
      ln=lm+nloop
      lo=ln+nloop
      lp=lo+nloop*6

      if(lp.gt.nreal) stop 'twoemm ds real'

      call chrgds(ncpods,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jc),iwkvec(jb),iwkvec(jd),g,coord,cdfitc,
     &            alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jm),wkvec(jq),
     &            wkvec(jr),wkvec(js),wkvec(ka),wkvec(kb),wkvec(kc),
     &            wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),wkvec(kh),
     &            wkvec(ki),wkvec(kj),wkvec(kk),wkvec(kl),wkvec(km),
     &            wkvec(kn),wkvec(ko),wkvec(kp),wkvec(kq),wkvec(kr),
     &            wkvec(ks),wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &            wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),wkvec(lb),
     &            wkvec(lc),wkvec(ld),wkvec(le),wkvec(lf),wkvec(lg),
     &            wkvec(lh),wkvec(li),wkvec(lj),wkvec(lk),wkvec(ll),
     &            wkvec(lm),wkvec(ln),wkvec(lo))

      ka=jt+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop*3
      kj=ki+nloop*6
      kk=kj+nloop*6
      kl=kk+nloop*10
      km=kl+nloop*10
      kn=km+nloop*15
      ko=kn+nloop*15
      kp=ko+nloop*18
      kq=kp+nloop*30
      kr=kq+nloop*36
      ks=kr+nloop
      kt=ks+nloop*3
      ku=kt+nloop*6
      kv=ku+nloop
      kw=kv+nloop
      kx=kw+nloop
      ky=kx+nloop
      kz=ky+nloop
      la=kz+nloop
      lb=la+nloop
      lc=lb+nloop
      ld=lc+nloop
      le=ld+nloop
      lf=le+nloop*18

      if(nxcfunc.gt.0) then
      
                 if(lf.gt.nreal) stop 'twoemm ds real'

                 call xcords(ncpods,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jc),iwkvec(jb),
     &                       iwkvec(jd),coord,xcfit1,xcfit2,xcfit3,
     &                       alphaxc,coefsxc,coefpxc,coefdxc,fock1,
     &                       fock2,fock3,wkvec(je),wkvec(jh),wkvec(ji),
     &                       wkvec(jj),wkvec(jn),wkvec(jo),wkvec(jp),
     &                       wkvec(jm),wkvec(ka),wkvec(kb),wkvec(kc),
     &                       wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),
     &                       wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &                       wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &                       wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),
     &                       wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &                       wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),
     &                       wkvec(lb),wkvec(lc),wkvec(ld),wkvec(le))

                       endif

      ja=1
      jb=ja+nppodp
      jc=jb+ncpodp*18

      if(jc.gt.ninteger) stop 'twoemm dp integer overflow'

      je=1
      jf=je+nppodp
      jg=jf+nppodp
      jh=jg+nppodp
      ji=jh+nppodp
      jj=ji+nppodp
      jk=jj+nppodp
      jl=jk+nppodp
      jm=jl+nppodp
      jn=jm+nppodp
      jo=jn+nppodp
      jp=jo+nppodp
      jq=jp+nppodp
      jr=jq+nppodp
      js=jr+nppodp
      jt=js+nppodp
      ju=jt+nppodp
      jv=ju+nppodp
      jw=jv+nppodp
      jx=jw+ncpodp
      jy=jx+ncpodp
      jz=jy+ncpodp
      j0=jz+nppodp*11

      if(j0.gt.nreal) stop 'twoemm dp real overflow'

      call setupcddp(ncpodp,n,ncontd,ncontp,nsheld,nshelp,mtloca,
     &               iprimvc,nprimvc,icentvc,iwkvec(ja),iwkvec(jb),
     &               cutoff,coord,alpha,coeff,wkvec(je),wkvec(jf),
     &               wkvec(jg),wkvec(jh),wkvec(ji),wkvec(jj),wkvec(jk),
     &               wkvec(jl),wkvec(jm),wkvec(jn),wkvec(jo),wkvec(jp),
     &               wkvec(jq),wkvec(jr),wkvec(js),wkvec(jt),wkvec(ju),
     &               wkvec(jv),wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jz),
     &               density)

      jd=jc+n
      j0=jd+n

      if(j0.gt.ninteger) stop 'twoemm dp integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                          call stringcopy(n,wkvec(12*n+1),wkvec(jq))
                          call stringcopy(n,wkvec(13*n+1),wkvec(jr))
                          call stringcopy(n,wkvec(14*n+1),wkvec(js))
                          call stringcopy(n,wkvec(15*n+1),wkvec(jt))
                          call stringcopy(n,wkvec(16*n+1),wkvec(ju))
                          call stringcopy(n,wkvec(17*n+1),wkvec(jv))
                    endif

      call stringcopy(ncpodp,wkvec(18*n+0*ncpodp+1),wkvec(jw))
      call stringcopy(ncpodp,wkvec(18*n+1*ncpodp+1),wkvec(jx))
      call stringcopy(ncpodp,wkvec(18*n+2*ncpodp+1),wkvec(jy))

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n
      jr=jq+n
      js=jr+n
      jt=js+n
      ju=jt+n
      jv=ju+n
      jw=jv+n
      jx=jw+ncpodp
      jy=jx+ncpodp
      jz=jy+ncpodp

      if(myid.eq.0) then
                          isent=0

                          do 1012 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=ja
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp
                          jsend(14)=jq
                          jsend(15)=jr
                          jsend(16)=js
                          jsend(17)=jt
                          jsend(18)=ju
                          jsend(19)=jv

                          call mpi_bsend(jsend,19,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(ja),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jq),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jr),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(js),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          ja=ja+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop
                          jq=jq+nloop
                          jr=jr+nloop
                          js=js+nloop
                          jt=jt+nloop
                          ju=ju+nloop
                          jv=jv+nloop

 1012                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,19,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ja=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)
      jq=jsend(14)
      jr=jsend(15)
      js=jsend(16)
      jt=jsend(17)
      ju=jsend(18)
      jv=jsend(19)

      call mpi_recv(iwkvec(ja),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jq),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jr),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(js),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+nloop
      kl=kk+nloop
      km=kl+nloop
      kn=km+nloop
      ko=kn+nloop
      kp=ko+nloop*3
      kq=kp+nloop*3
      kr=kq+nloop*3
      ks=kr+nloop*3
      kt=ks+nloop*3
      ku=kt+nloop*6
      kv=ku+nloop*6
      kw=kv+nloop*6
      kx=kw+nloop*6
      ky=kx+nloop*6
      kz=ky+nloop*10
      la=kz+nloop*10
      lb=la+nloop*10
      lc=lb+nloop*10
      ld=lc+nloop*9
      le=ld+nloop*18
      lf=le+nloop*18
      lg=lf+nloop*18
      lh=lg+nloop*30
      li=lh+nloop*30
      lj=li+nloop*30
      lk=lj+nloop*36
      ll=lk+nloop*36
      lm=ll+nloop*60
      ln=lm+nloop*60
      lo=ln+nloop*18
      lp=lo+nloop*54
      lq=lp+nloop*108
      lr=lq+nloop
      ls=lr+nloop
      lt=ls+nloop
      lu=lt+nloop*3
      lv=lu+nloop*6
      lw=lv+nloop*9
      lx=lw+nloop*10
      ly=lx+nloop*18
      lz=ly+nloop
      ma=lz+nloop
      mb=ma+nloop
      mc=mb+nloop
      md=mc+nloop
      me=md+nloop
      mf=me+nloop
      mg=mf+nloop*18

      if(mg.gt.nreal) stop 'twoemm dp real'

      call chrgdp(ncpodp,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jc),iwkvec(jb),iwkvec(jd),g,coord,cdfitc,
     &            alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jn),wkvec(jo),
     &            wkvec(jp),wkvec(jm),wkvec(jq),wkvec(jr),wkvec(js),
     &            wkvec(ka),wkvec(kb),wkvec(kc),wkvec(kd),wkvec(ke),
     &            wkvec(kf),wkvec(kg),wkvec(kh),wkvec(ki),wkvec(kj),
     &            wkvec(kk),wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &            wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),wkvec(kt),
     &            wkvec(ku),wkvec(kv),wkvec(kw),wkvec(kx),wkvec(ky),
     &            wkvec(kz),wkvec(la),wkvec(lb),wkvec(lc),wkvec(ld),
     &            wkvec(le),wkvec(lf),wkvec(lg),wkvec(lh),wkvec(li),
     &            wkvec(lj),wkvec(lk),wkvec(ll),wkvec(lm),wkvec(ln),
     &            wkvec(lo),wkvec(lp),wkvec(lq),wkvec(lr),wkvec(ls),
     &            wkvec(lt),wkvec(lu),wkvec(lv),wkvec(lw),wkvec(lx),
     &            wkvec(ly),wkvec(lz),wkvec(ma),wkvec(mb),wkvec(mc),
     &            wkvec(md),wkvec(me),wkvec(mf))

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop*3
      kj=ki+nloop*6
      kk=kj+nloop*6
      kl=kk+nloop*10
      km=kl+nloop*10
      kn=km+nloop*15
      ko=kn+nloop*15
      kp=ko+nloop*21
      kq=kp+nloop*21
      kr=kq+nloop*18
      ks=kr+nloop*30
      kt=ks+nloop*45
      ku=kt+nloop*54
      kv=ku+nloop*90
      kw=kv+nloop*108
      kx=kw+nloop
      ky=kx+nloop*3
      kz=ky+nloop*6
      la=kz+nloop*10
      lb=la+nloop
      lc=lb+nloop
      ld=lc+nloop
      le=ld+nloop
      lf=le+nloop
      lg=lf+nloop
      lh=lg+nloop
      li=lh+nloop
      lj=li+nloop
      lk=lj+nloop
      ll=lk+nloop*54

      if(nxcfunc.gt.0) then
   
                 if(ll.gt.nreal) stop 'twoemm dp real'

                 call xcordp(ncpodp,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jc),iwkvec(jb),
     &                       iwkvec(jd),coord,xcfit1,xcfit2,xcfit3,
     &                       alphaxc,coefsxc,coefpxc,coefdxc,fock1,
     &                       fock2,fock3,wkvec(je),wkvec(jh),wkvec(ji),
     &                       wkvec(jj),wkvec(jw),wkvec(jx),wkvec(jy),
     &                       wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jm),
     &                       wkvec(ka),wkvec(kb),wkvec(kc),wkvec(kd),
     &                       wkvec(ke),wkvec(kf),wkvec(kg),wkvec(kh),
     &                       wkvec(ki),wkvec(kj),wkvec(kk),wkvec(kl),
     &                       wkvec(km),wkvec(kn),wkvec(ko),wkvec(kp),
     &                       wkvec(kq),wkvec(kr),wkvec(ks),wkvec(kt),
     &                       wkvec(ku),wkvec(kv),wkvec(kw),wkvec(kx),
     &                       wkvec(ky),wkvec(kz),wkvec(la),wkvec(lb),
     &                       wkvec(lc),wkvec(ld),wkvec(le),wkvec(lf),
     &                       wkvec(lg),wkvec(lh),wkvec(li),wkvec(lj),
     &                       wkvec(lk))

                       endif

      ja=1
      jb=ja+ncpodd
      jc=jb+nppodd
      jd=jc+ncpodd*36

      if(jd.gt.ninteger) stop 'twoemm dd integer overflow'

      je=1
      jf=je+nppodd
      jg=jf+nppodd
      jh=jg+nppodd
      ji=jh+nppodd
      jj=ji+nppodd
      jk=jj+nppodd
      jl=jk+nppodd
      jm=jl+nppodd
      jn=jm+nppodd
      jo=jn+nppodd
      jp=jo+nppodd
      jq=jp+nppodd
      jr=jq+nppodd
      js=jr+nppodd
      jt=js+nppodd
      ju=jt+nppodd
      jv=ju+nppodd
      jw=jv+nppodd
      jx=jw+ncpodd
      jy=jx+ncpodd
      jz=jy+ncpodd
      j0=jz+nppodd*11

      if(j0.gt.nreal) stop 'twoemm dd real overflow'

      call setupcddd(ncpodd,n,ncontd,nsheld,mtloca,iprimvc,nprimvc,
     &               icentvc,iwkvec(ja),iwkvec(jb),iwkvec(jc),cutoff,
     &               coord,alpha,coeff,wkvec(je),wkvec(jf),wkvec(jg),
     &               wkvec(jh),wkvec(ji),wkvec(jj),wkvec(jk),wkvec(jl),
     &               wkvec(jm),wkvec(jn),wkvec(jo),wkvec(jp),wkvec(jq),
     &               wkvec(jr),wkvec(js),wkvec(jt),wkvec(ju),wkvec(jv),
     &               wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jz),density)

      j0=jd+n
      j1=j0+n

      if(j1.gt.ninteger) stop 'twoemm dd integer overflow'

      if(myid.eq.0) then
                          call stringcopy(n,wkvec(00*n+1),wkvec(je))
                          call stringcopy(n,wkvec(01*n+1),wkvec(jf))
                          call stringcopy(n,wkvec(02*n+1),wkvec(jg))
                          call stringcopy(n,wkvec(03*n+1),wkvec(jh))
                          call stringcopy(n,wkvec(04*n+1),wkvec(ji))
                          call stringcopy(n,wkvec(05*n+1),wkvec(jj))
                          call stringcopy(n,wkvec(06*n+1),wkvec(jk))
                          call stringcopy(n,wkvec(07*n+1),wkvec(jl))
                          call stringcopy(n,wkvec(08*n+1),wkvec(jm))
                          call stringcopy(n,wkvec(09*n+1),wkvec(jn))
                          call stringcopy(n,wkvec(10*n+1),wkvec(jo))
                          call stringcopy(n,wkvec(11*n+1),wkvec(jp))
                          call stringcopy(n,wkvec(12*n+1),wkvec(jq))
                          call stringcopy(n,wkvec(13*n+1),wkvec(jr))
                          call stringcopy(n,wkvec(14*n+1),wkvec(js))
                          call stringcopy(n,wkvec(15*n+1),wkvec(jt))
                          call stringcopy(n,wkvec(16*n+1),wkvec(ju))
                          call stringcopy(n,wkvec(17*n+1),wkvec(jv))
                    endif

      call stringcopy(ncpodd,wkvec(18*n+0*ncpodd+1),wkvec(jw))
      call stringcopy(ncpodd,wkvec(18*n+1*ncpodd+1),wkvec(jx))
      call stringcopy(ncpodd,wkvec(18*n+2*ncpodd+1),wkvec(jy))

      je=1
      jf=je+n
      jg=jf+n
      jh=jg+n
      ji=jh+n
      jj=ji+n
      jk=jj+n
      jl=jk+n
      jm=jl+n
      jn=jm+n
      jo=jn+n
      jp=jo+n
      jq=jp+n
      jr=jq+n
      js=jr+n
      jt=js+n
      ju=jt+n
      jv=ju+n
      jw=jv+n
      jx=jw+ncpodd
      jy=jx+ncpodd
      jz=jy+ncpodd

      if(myid.eq.0) then
                          isent=0

                          do 1013 islave=0,numprocs-1

                          ia=n-isent
                          ib=numprocs-islave

                          nloop=ia/ib
                          if(mod(ia,ib).ne.0) nloop=nloop+1

                          call mpi_bsend(nloop,1,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          jsend(01)=jb
                          jsend(02)=je
                          jsend(03)=jf
                          jsend(04)=jg
                          jsend(05)=jh
                          jsend(06)=ji
                          jsend(07)=jj
                          jsend(08)=jk
                          jsend(09)=jl
                          jsend(10)=jm
                          jsend(11)=jn
                          jsend(12)=jo
                          jsend(13)=jp
                          jsend(14)=jq
                          jsend(15)=jr
                          jsend(16)=js
                          jsend(17)=jt
                          jsend(18)=ju
                          jsend(19)=jv

                          call mpi_bsend(jsend,19,mpi_integer,islave,
     &                                   islave,mpi_comm_world,ierr)

                          call mpi_bsend(iwkvec(jb),nloop,
     &                                   mpi_integer,islave,islave,
     &                                   mpi_comm_world,ierr)

                          call mpi_bsend(wkvec(je),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jh),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(ji),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jj),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jm),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jn),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jo),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jp),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jq),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(jr),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)
                          call mpi_bsend(wkvec(js),nloop,
     &                                   mpi_real8,islave,islave,
     &                                   mpi_comm_world,ierr)

                          jb=jb+nloop

                          je=je+nloop
                          jf=jf+nloop
                          jg=jg+nloop
                          jh=jh+nloop
                          ji=ji+nloop
                          jj=jj+nloop
                          jk=jk+nloop
                          jl=jl+nloop
                          jm=jm+nloop
                          jn=jn+nloop
                          jo=jo+nloop
                          jp=jp+nloop
                          jq=jq+nloop
                          jr=jr+nloop
                          js=js+nloop
                          jt=jt+nloop
                          ju=ju+nloop
                          jv=jv+nloop

 1013                     isent=isent+nloop
                    endif

      call mpi_recv(nloop,1,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(jsend,19,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      jb=jsend(01)
      je=jsend(02)
      jf=jsend(03)
      jg=jsend(04)
      jh=jsend(05)
      ji=jsend(06)
      jj=jsend(07)
      jk=jsend(08)
      jl=jsend(09)
      jm=jsend(10)
      jn=jsend(11)
      jo=jsend(12)
      jp=jsend(13)
      jq=jsend(14)
      jr=jsend(15)
      js=jsend(16)
      jt=jsend(17)
      ju=jsend(18)
      jv=jsend(19)

      call mpi_recv(iwkvec(jb),nloop,mpi_integer,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      call mpi_recv(wkvec(je),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jh),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(ji),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jj),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jm),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jn),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jo),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jp),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jq),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(jr),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)
      call mpi_recv(wkvec(js),nloop,mpi_real8,mpi_any_source,
     &              mpi_any_tag,mpi_comm_world,istat,ierror)

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop
      kj=ki+nloop
      kk=kj+nloop
      kl=kk+nloop
      km=kl+nloop
      kn=km+nloop
      ko=kn+nloop
      kp=ko+nloop
      kq=kp+nloop*3
      kr=kq+nloop*3
      ks=kr+nloop*3
      kt=ks+nloop*3
      ku=kt+nloop*3
      kv=ku+nloop*3
      kw=kv+nloop*6
      kx=kw+nloop*6
      ky=kx+nloop*6
      kz=ky+nloop*6
      la=kz+nloop*6
      lb=la+nloop*6
      lc=lb+nloop*10
      ld=lc+nloop*10
      le=ld+nloop*10
      lf=le+nloop*10
      lg=lf+nloop*10
      lh=lg+nloop*15
      li=lh+nloop*15
      lj=li+nloop*15
      lk=lj+nloop*15
      ll=lk+nloop*9
      lm=ll+nloop*9
      ln=lm+nloop*18
      lo=ln+nloop*18
      lp=lo+nloop*18
      lq=lp+nloop*30
      lr=lq+nloop*30
      ls=lr+nloop*30
      lt=ls+nloop*45
      lu=lt+nloop*45
      lv=lu+nloop*45
      lw=lv+nloop*36
      lx=lw+nloop*36
      ly=lx+nloop*60
      lz=ly+nloop*60
      ma=lz+nloop*90
      mb=ma+nloop*90
      mc=mb+nloop*18
      md=mc+nloop*30
      me=md+nloop*36
      mf=me+nloop*54
      mg=mf+nloop*90
      mh=mg+nloop*108
      mi=mh+nloop*108
      mj=mi+nloop*180
      mk=mj+nloop*216
      ml=mk+nloop
      mm=ml+nloop
      mn=mm+nloop
      mo=mn+nloop*3
      mp=mo+nloop*6
      mq=mp+nloop*10
      mr=mq+nloop*15
      ms=mr+nloop*30
      mt=ms+nloop
      mu=mt+nloop
      mv=mu+nloop
      mw=mv+nloop
      mx=mw+nloop
      my=mx+nloop
      mz=my+nloop
      na=mz+nloop*36

      if(na.gt.nreal) stop 'twoemm dd real'

      call chrgdd(ncpodd,nloop,ncds,ncdspd,icdlfunc,icdcfunc,iwkvec(ja),
     &            iwkvec(jb),iwkvec(jd),iwkvec(jc),iwkvec(j0),g,coord,
     &            cdfitc,alphacd,coefscd,coefpcd,coefdcd,fock,wkvec(je),
     &            wkvec(jw),wkvec(jx),wkvec(jy),wkvec(jn),wkvec(jo),
     &            wkvec(jp),wkvec(jm),wkvec(jq),wkvec(jr),wkvec(js),
     &            wkvec(ka),wkvec(kb),wkvec(kc),wkvec(kd),wkvec(ke),
     &            wkvec(kf),wkvec(kg),wkvec(kh),wkvec(ki),wkvec(kj),
     &            wkvec(kk),wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &            wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),wkvec(kt),
     &            wkvec(ku),wkvec(kv),wkvec(kw),wkvec(kx),wkvec(ky),
     &            wkvec(kz),wkvec(la),wkvec(lb),wkvec(lc),wkvec(ld),
     &            wkvec(le),wkvec(lf),wkvec(lg),wkvec(lh),wkvec(li),
     &            wkvec(lj),wkvec(lk),wkvec(ll),wkvec(lm),wkvec(ln),
     &            wkvec(lo),wkvec(lp),wkvec(lq),wkvec(lr),wkvec(ls),
     &            wkvec(lt),wkvec(lu),wkvec(lv),wkvec(lw),wkvec(lx),
     &            wkvec(ly),wkvec(lz),wkvec(ma),wkvec(mb),wkvec(mc),
     &            wkvec(md),wkvec(me),wkvec(mf),wkvec(mg),wkvec(mh),
     &            wkvec(mi),wkvec(mj),wkvec(mk),wkvec(ml),wkvec(mm),
     &            wkvec(mn),wkvec(mo),wkvec(mp),wkvec(mq),wkvec(mr),
     &            wkvec(ms),wkvec(mt),wkvec(mu),wkvec(mv),wkvec(mw),
     &            wkvec(mx),wkvec(my),wkvec(mz))

      ka=jz+1
      kb=ka+nloop
      kc=kb+nloop
      kd=kc+nloop
      ke=kd+nloop
      kf=ke+nloop
      kg=kf+nloop
      kh=kg+nloop
      ki=kh+nloop*3
      kj=ki+nloop*6
      kk=kj+nloop*6
      kl=kk+nloop*10
      km=kl+nloop*10
      kn=km+nloop*15
      ko=kn+nloop*15
      kp=ko+nloop*21
      kq=kp+nloop*21
      kr=kq+nloop*28
      ks=kr+nloop*28
      kt=ks+nloop*18
      ku=kt+nloop*36
      kv=ku+nloop*108
      kw=kv+nloop*216
      kx=kw+nloop*30
      ky=kx+nloop*60
      kz=ky+nloop*180
      la=kz+nloop*45
      lb=la+nloop*90
      lc=lb+nloop*63
      ld=lc+nloop
      le=ld+nloop*3
      lf=le+nloop*6
      lg=lf+nloop*10
      lh=lg+nloop*15
      li=lh+nloop
      lj=li+nloop
      lk=lj+nloop
      ll=lk+nloop
      lm=ll+nloop
      ln=lm+nloop
      lo=ln+nloop
      lp=lo+nloop
      lq=lp+nloop
      lr=lq+nloop
      ls=lr+nloop*108

      if(nxcfunc.gt.0) then

                 if(ls.gt.nreal) stop 'twoemm dd real'

                 call xcordd(ncpodd,nloop,nxcs,nxcspd,ixclfunc,ixccfunc,
     &                       iwkvec(ja),iwkvec(jb),iwkvec(jd),
     &                       iwkvec(jc),iwkvec(j0),coord,xcfit1,xcfit2,
     &                       xcfit3,alphaxc,coefsxc,coefpxc,coefdxc,
     &                       fock1,fock2,fock3,wkvec(je),wkvec(jh),
     &                       wkvec(ji),wkvec(jj),wkvec(jw),wkvec(jx),
     &                       wkvec(jy),wkvec(jn),wkvec(jo),wkvec(jp),
     &                       wkvec(jm),wkvec(ka),wkvec(kb),wkvec(kc),
     &                       wkvec(kd),wkvec(ke),wkvec(kf),wkvec(kg),
     &                       wkvec(kh),wkvec(ki),wkvec(kj),wkvec(kk),
     &                       wkvec(kl),wkvec(km),wkvec(kn),wkvec(ko),
     &                       wkvec(kp),wkvec(kq),wkvec(kr),wkvec(ks),
     &                       wkvec(kt),wkvec(ku),wkvec(kv),wkvec(kw),
     &                       wkvec(kx),wkvec(ky),wkvec(kz),wkvec(la),
     &                       wkvec(lb),wkvec(lc),wkvec(ld),wkvec(le),
     &                       wkvec(lf),wkvec(lg),wkvec(lh),wkvec(li),
     &                       wkvec(lj),wkvec(lk),wkvec(ll),wkvec(lm),
     &                       wkvec(ln),wkvec(lo),wkvec(lp),wkvec(lq),
     &                       wkvec(lr))

                       endif

 2001 continue

      call mpi_reduce(fock,wkvec,ndim,mpi_real8,
     &                mpi_sum,0,mpi_comm_world,ierr)

      if(myid.eq.0) call stringcopy(ndim,fock,wkvec)

      call mpi_reduce(fock1,wkvec,ndim,mpi_real8,
     &                mpi_sum,0,mpi_comm_world,ierr)

      if(myid.eq.0) call stringcopy(ndim,fock1,wkvec)

      call mpi_reduce(fock2,wkvec,ndim,mpi_real8,
     &                mpi_sum,0,mpi_comm_world,ierr)

      if(myid.eq.0) call stringcopy(ndim,fock2,wkvec)

      call mpi_reduce(fock3,wkvec,ndim,mpi_real8,
     &                mpi_sum,0,mpi_comm_world,ierr)

      if(myid.eq.0) call stringcopy(ndim,fock3,wkvec)

      if(myid.eq.0) then
                          call normd(nconts,ncontp,ncontd,
     &                               nshels,nshelp,nsheld,mtloca,fock)

              if(nxcfunc.gt.0) then

                          call normd(nconts,ncontp,ncontd,
     &                               nshels,nshelp,nsheld,mtloca,fock3)

                          do 1014 i=1,ndim
                          fock1(i)=fock1(i)+fock3(i)
 1014                     fock2(i)=fock2(i)+fock3(i)

                               endif

                          do 1015 i=1,ndim
                          fock1(i)=fock(i)+fock1(i)
                          fock2(i)=fock(i)+fock2(i)
 1015                     fock3(i)=fock(i)+fock3(i)
                    endif

      call mpi_bcast(fock1,ndim,mpi_real8,0,mpi_comm_world,ierr)
      call mpi_bcast(fock2,ndim,mpi_real8,0,mpi_comm_world,ierr)
      call mpi_bcast(fock3,ndim,mpi_real8,0,mpi_comm_world,ierr)

      return
      end
